@@ -40,6 +40,48 @@ static void print_uuid(const u8 *uuid)
 	}
 }
 
+static void print_escaped(const char *str)
+{
+	while (*str) {
+		switch (*str) {
+		case '\b':			/* 0x08 */
+			putchar('\\');
+			putchar('b');
+			break;
+		case '\t':			/* 0x09 */
+			putchar('\\');
+			putchar('t');
+			break;
+		case '\n':			/* 0x0a */
+			putchar('\\');
+			putchar('n');
+			break;
+		case '\f':			/* 0x0c */
+			putchar('\\');
+			putchar('f');
+			break;
+		case '\r':			/* 0x0d */
+			putchar('\\');
+			putchar('r');
+			break;
+		/* Other control characters from 0 .. 31 */
+		case '\v':			/* 0x0b */
+		case 0x00 ... 0x07:
+		case 0x0e ... 0x1f:
+			printf("\\u%04x", *str);
+			break;
+		/* '/' (solidus) not escaped */
+		case '"':
+		case '\\':
+			putchar('\\');
+			fallthrough;
+		default:
+			putchar(*str);
+		}
+		str++;
+	}
+}
+
 static void fmt_indent1(int indent)
 {
 	while (indent--)
@@ -272,7 +314,8 @@ void fmt_print(struct format_ctx *fctx, const char* key, ...)
 		} else {
 			/* Simple key/values */
 			fmt_separator(fctx);
-			printf("\"%s\": ", row->out_json);
+			if (row->out_json)
+				printf("\"%s\": ", row->out_json);
 		}
 	}
 
@@ -280,6 +323,10 @@ void fmt_print(struct format_ctx *fctx, const char* key, ...)
 
 	if (row->fmt[0] == '%') {
 		vprintf(row->fmt, args);
+	} else if (strcmp(row->fmt, "str") == 0) {
+		const char *str = va_arg(args, const char *);
+
+		print_escaped(str);
 	} else if (strcmp(row->fmt, "uuid") == 0) {
 		const u8 *uuid = va_arg(args, const u8*);
 
